from os.path import join

import pytest
from hdx.location.country import Country
from hdx.utilities.loader import load_yaml
from hdx.utilities.text import multiple_replace
from slugify import slugify

from hdx.api.configuration import Configuration
from hdx.api.locations import Locations
from hdx.data.dataset import Dataset
from hdx.data.resource import Resource
from hdx.data.vocabulary import Vocabulary


class TestUpdateLogic:
    @pytest.fixture(scope="function")
    def configuration(self):
        Configuration._create(
            hdx_read_only=True,
            user_agent="test",
            project_config_yaml=join(
                "tests", "fixtures", "config", "project_configuration.yml"
            ),
        )
        Locations.set_validlocations([{"name": "zmb", "title": "Zambia"}])
        Country.countriesdata(use_live=False)
        Vocabulary._tags_dict = dict()
        Vocabulary._approved_vocabulary = {
            "tags": [
                {"name": "hxl"},
                {"name": "indicators"},
                {"name": "health"},
                {"name": "demographics"},
                {"name": "sustainable development goals - sdg"},
            ],
            "id": "4e61d464-4943-4e97-973a-84673c1aaa87",
            "name": "approved",
        }

    @pytest.fixture(scope="class")
    def fixture_path(self):
        return join("tests", "fixtures", "update_logic")

    @pytest.fixture(scope="class")
    def new_resources_yaml(self, fixture_path):
        return join(fixture_path, "update_logic_resources_new.yml")

    @pytest.fixture(scope="class")
    def resources_yaml(self, fixture_path):
        return join(fixture_path, "update_logic_resources.yml")

    @pytest.fixture(scope="class")
    def dataset_data(self):
        return {
            "name": "who-data-for-zambia",
            "title": "Zambia - Health Indicators",
            "private": False,
            "notes": "Contains data from World Health Organization...",
            "dataset_source": "World Health Organization",
            "license_id": "hdx-other",
            "license_other": "CC BY-NC-SA 3.0 IGO",
            "methodology": "Registry",
        }

    @pytest.fixture(scope="function")
    def new_dataset(self, dataset_data):
        new_dataset = Dataset(dataset_data)
        new_dataset.set_maintainer("35f7bb2c-4ab6-4796-8334-525b30a94c89")
        new_dataset.set_organization("c021f6be-3598-418e-8f7f-c7a799194dba")
        new_dataset.set_expected_update_frequency("Every month")
        new_dataset.set_subnational(False)
        new_dataset.set_dataset_year_range(1961, 2019)
        new_dataset.add_country_location("zmb")
        new_dataset.add_tag("hxl")
        return new_dataset

    @pytest.fixture(scope="function")
    def dataset(self, dataset_data, resources_yaml):
        dataset = Dataset(dataset_data)
        dataset.set_maintainer("35f7bb2c-4ab6-4796-8334-525b30a94c89")
        dataset.set_organization("c021f6be-3598-418e-8f7f-c7a799194dba")
        dataset.set_expected_update_frequency("Every month")
        dataset.set_subnational(False)
        dataset.set_dataset_year_range(1961, 2019)
        dataset.add_country_location("zmb")
        dataset.add_tag("hxl")
        dataset["id"] = "3adc4bb0-faef-42ae-bd67-0ea08918a629"
        return dataset

    @staticmethod
    def add_dataset_resources(dataset, yaml_path, include=None, exclude=None):
        resources = load_yaml(yaml_path)
        if include is None:
            include = range(len(resources))
        if exclude is None:
            exclude = list()
        for i, resource in enumerate(resources):
            if i not in include:
                continue
            if i in exclude:
                continue
            dataset.add_update_resource(resource)

    @staticmethod
    def add_new_dataset_resources(
        new_dataset, yaml_path, include=None, exclude=None
    ):
        resources_uploads = load_yaml(yaml_path)
        if include is None:
            include = range(len(resources_uploads))
        if exclude is None:
            exclude = list()
        for i, resource_upload in enumerate(resources_uploads):
            if i not in include:
                continue
            if i in exclude:
                continue
            resource = resource_upload["resource"]
            file_to_upload = resource_upload["file_to_upload"]
            resource = Resource(resource)
            resource.set_file_to_upload(file_to_upload)
            new_dataset.add_update_resource(resource)

    @staticmethod
    def check_resources(update, results):
        resources = update["resources"]
        files_to_upload = results["files_to_upload"]
        for key in files_to_upload:
            index = key.replace("update__resources__", "")
            index = int(index.replace("__upload", ""))
            resource = resources[index]
            rn = resource["name"]
            name = slugify(rn, separator="_")
            name = multiple_replace(
                name,
                {
                    "all_health": "health",
                    "quickcharts": "qc_health",
                    "for_zambia": "ZMB.csv",
                },
            )
            name = f"/tmp/WHO/{name}"
            fu = files_to_upload[key]
            assert (
                name == fu
            ), f'Mismatch at index {index}: "{rn}", file to upload is "{fu}"'

    def test_update_logic_1(
        self,
        configuration,
        new_dataset,
        dataset,
        new_resources_yaml,
        resources_yaml,
    ):
        self.add_new_dataset_resources(
            new_dataset, new_resources_yaml, exclude=[13]
        )
        self.add_dataset_resources(dataset, resources_yaml)
        dataset.old_data = new_dataset.data
        dataset.old_data["resources"] = new_dataset.resources
        results = dataset._dataset_merge_hdx_update(
            update_resources=True,
            match_resources_by_metadata=True,
            keys_to_delete=list(),
            remove_additional_resources=True,
            match_resource_order=False,
            create_default_views=False,
            hxl_update=False,
            test=True,
        )
        assert results["filter"] == list()
        update = results["update"]
        del update["updated_by_script"]
        assert update == {
            "name": "who-data-for-zambia",
            "title": "Zambia - Health Indicators",
            "private": False,
            "notes": "Contains data from World Health Organization...",
            "dataset_source": "World Health Organization",
            "license_id": "hdx-other",
            "license_other": "CC BY-NC-SA 3.0 IGO",
            "methodology": "Registry",
            "maintainer": "35f7bb2c-4ab6-4796-8334-525b30a94c89",
            "owner_org": "c021f6be-3598-418e-8f7f-c7a799194dba",
            "data_update_frequency": "30",
            "subnational": "0",
            "dataset_date": "[1961-01-01T00:00:00 TO 2019-12-31T00:00:00]",
            "groups": [{"name": "zmb"}],
            "tags": [
                {
                    "name": "hxl",
                    "vocabulary_id": "4e61d464-4943-4e97-973a-84673c1aaa87",
                }
            ],
            "id": "3adc4bb0-faef-42ae-bd67-0ea08918a629",
            "resources": [
                {
                    "id": "1e7a68da-501a-444e-94f1-3606263e10c8",
                    "name": "All Health Indicators for Zambia",
                    "description": "See resource descriptions below for links to indicator metadata",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "40fbcfcc-e180-455d-9d65-a9f8d1f85643",
                    "name": "Mortality and global health estimates Indicators for Zambia",
                    "description": "*Mortality and global health estimates:*\n[Infant mortality rate (proba",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "018577c8-a397-4a4f-b13c-889c4990d5e2",
                    "name": "Sustainable development goals Indicators for Zambia",
                    "description": "*Sustainable development goals:*\n[Adolescent birth rate (per 1000 wome",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b42f3a3e-82a9-4fe6-9f0c-81d520ce7666",
                    "name": "Millennium Development Goals (MDGs) Indicators for Zambia",
                    "description": "*Millennium Development Goals (MDGs):*\n[Contraceptive prevalence (%)](",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b67738ca-812f-41b2-80e3-7c3340648398",
                    "name": "Health systems Indicators for Zambia",
                    "description": "*Health systems:*\n[Median availability of selected generic medicines (",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "a610cbb6-3bd2-4400-9aee-899dbe6b4a98",
                    "name": "Malaria Indicators for Zambia",
                    "description": "*Malaria:*\n[Children aged <5 years sleeping under insecticide-treated ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "de38982e-ef0d-4e5c-b0a3-16f933eaa808",
                    "name": "Tuberculosis Indicators for Zambia",
                    "description": "*Tuberculosis:*\n[Deaths due to tuberculosis among HIV-negative people ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "c330fc6e-624d-4d7f-bb46-f5ea67cf5d0a",
                    "name": "Child health Indicators for Zambia",
                    "description": "*Child health:*\n[Children aged <5 years stunted (%)](https://www.who.i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9903aea5-d193-4d9d-9018-e98213aef0f5",
                    "name": "Infectious diseases Indicators for Zambia",
                    "description": "*Infectious diseases:*\n[Cholera - number of reported cases](https://ww",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "fb24e4d1-9447-496d-9049-4712e4cd90c6",
                    "name": "World Health Statistics Indicators for Zambia",
                    "description": "*World Health Statistics:*\n[Literacy rate among adults aged >= 15 year",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "98e022a6-e054-41fd-af9a-e4c08ddcc5cd",
                    "name": "Health financing Indicators for Zambia",
                    "description": "*Health financing:*\n[Private prepaid plans as a percentage of private ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9a72db07-1bcc-4316-a427-1c208ebb5708",
                    "name": "Public health and environment Indicators for Zambia",
                    "description": "*Public health and environment:*\n[Population using solid fuels (%)](ht",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "873ead28-fff0-49d6-83c4-1ccda19cb0f7",
                    "name": "Substance use and mental health Indicators for Zambia",
                    "description": "*Substance use and mental health:*\n[Fines for violations](https://www.",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "74e93f31-494c-4950-b891-6db42537e8b4",
                    "name": "Injuries and violence Indicators for Zambia",
                    "description": "*Injuries and violence:*\n[Income level](https://www.who.int/data/gho/i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ef319875-76fb-4ff4-bddb-02715a7638df",
                    "name": "HIV/AIDS and other STIs Indicators for Zambia",
                    "description": "*HIV/AIDS and other STIs:*\n[Prevalence of HIV among adults aged 15 to ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1d4b9ac1-71ed-4ed3-a120-c4c6fea00a3d",
                    "name": "Nutrition Indicators for Zambia",
                    "description": "*Nutrition:*\n[Early initiation of breastfeeding (%)](https://www.who.i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1815f97a-d93c-4e64-909d-39f288880c50",
                    "name": "Urban health Indicators for Zambia",
                    "description": "*Urban health:*\n[Percentage of the total population living in cities >",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "23af66cb-cffb-491f-a51b-c41584b83f3b",
                    "name": "Noncommunicable diseases Indicators for Zambia",
                    "description": "*Noncommunicable diseases:*\n[Prevalence of overweight among adults, BM",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2beb31ac-873d-47f4-9cfc-75cebd2303b5",
                    "name": "Noncommunicable diseases CCS Indicators for Zambia",
                    "description": "*Noncommunicable diseases CCS:*\n[Existence of operational policy/strat",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9a14881d-196a-44dc-bcf6-bda199b1137b",
                    "name": "Negelected tropical diseases Indicators for Zambia",
                    "description": "*Negelected tropical diseases:*\n[Number of new reported cases of Burul",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "79b0d05f-9180-492c-b614-687a9203bc1c",
                    "name": "Health Equity Monitor Indicators for Zambia",
                    "description": "*Health Equity Monitor:*\n[Antenatal care coverage - at least one visit",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "4b4defd7-ce45-46e3-86ce-15154be90d0c",
                    "name": "Infrastructure Indicators for Zambia",
                    "description": "*Infrastructure:*\n[Total density per 100 000 population: Health posts]",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "29ac1338-b7b5-429f-bb38-455a66e8f6c1",
                    "name": "Essential health technologies Indicators for Zambia",
                    "description": "*Essential health technologies:*\n[Availability of national standards o",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "63ba2693-fe73-4555-9c64-93710f7c7404",
                    "name": "Medical equipment Indicators for Zambia",
                    "description": "*Medical equipment:*\n[Total density per million population: Magnetic R",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2cdc3198-4ec7-4a25-86ee-8cdc80b670fc",
                    "name": "Demographic and socioeconomic statistics Indicators for Zambia",
                    "description": "*Demographic and socioeconomic statistics:*\n[Cellular subscribers (per",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1028087c-af76-4ab2-ac30-0babb4ef00fc",
                    "name": "Neglected tropical diseases Indicators for Zambia",
                    "description": "*Neglected tropical diseases:*\n[Status of yaws endemicity](https://www",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "0ec8b0b2-1e0d-4338-b22a-e984f8d07f0a",
                    "name": "International Health Regulations (2005) monitoring framework Indicators for Zambia",
                    "description": "*International Health Regulations (2005) monitoring framework:*\n[Legis",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2fa6877b-5b6d-446f-9ccf-52e2478d86da",
                    "name": "Insecticide resistance Indicators for Zambia",
                    "description": "*Insecticide resistance:*\n[Number of insecticide classes to which resi",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "262fddcb-62eb-42f1-94c6-ed20e6b18650",
                    "name": "Universal Health Coverage Indicators for Zambia",
                    "description": "*Universal Health Coverage:*\n[Cataract surgical coverage of adults age",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5562627b-b7af-4d4d-a9ca-e4fcd996a180",
                    "name": "Global Observatory for eHealth (GOe) Indicators for Zambia",
                    "description": "*Global Observatory for eHealth (GOe):*\n[National universal health cov",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "53a1eeda-7a53-4b8c-87c8-c2ea276a52a3",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING : PREVENTION Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING : PREVENTION:*\n[Government uni",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "64c1aa0d-fa9a-4387-812e-fa7d00f382c2",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING: TREATMENT Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING: TREATMENT:*\n[Government unit/",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5b82a27c-a461-475b-867b-0afdc93132fd",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING: FINANCING Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING: FINANCING:*\n[Five-year change",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "f0f95f26-0701-49ee-a9ac-d1525572f6a5",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT SECTORS AND PROVIDERS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT SECTORS AND PROVID",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9b805d00-3de4-48f2-a93f-d859961e9c08",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT CAPACITY AND TREATMENT COVERAGE Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT CAPACITY AND TREAT",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9667ee27-97e5-4122-9dc6-e39df70eeb08",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: PHARMACOLOGICAL TREATMENT Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: PHARMACOLOGICAL TREATMENT:*\n",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5e16b87e-5a06-4f7f-badb-3a2e44d64948",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: SCREENING AND BRIEF INTERVENTIONS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: SCREENING AND BRIEF INTERVEN",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "d79c3183-471f-4331-94d6-13474171be91",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: PREVENTION PROGRAMS AND PROVIDERS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: PREVENTION PROGRAMS AND PROV",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "daa99f76-d84b-4c05-b5dc-c9e82fbcb859",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: SPECIAL PROGRAMMES AND SERVICES Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: SPECIAL PROGRAMMES AND SERVI",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ca1774e9-49c0-42a2-93c0-88fa32bb5adc",
                    "name": "RSUD: HUMAN RESOURCES Indicators for Zambia",
                    "description": "*RSUD: HUMAN RESOURCES:*\n[Health professionals providing treatment for",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "d244efc2-aa38-4197-b594-9bbd2b44bd9a",
                    "name": "RSUD: INFORMATION SYSTEMS Indicators for Zambia",
                    "description": "*RSUD: INFORMATION SYSTEMS:*\n[Epidemiological data collection for subs",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ee06d52b-5c15-4a0c-8d00-830215aeff03",
                    "name": "RSUD: YOUTH Indicators for Zambia",
                    "description": "*RSUD: YOUTH:*\n[Epidemiological data collection system for substance u",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1ef02dd7-07cc-44f9-83ee-fc945f6d22ea",
                    "name": "FINANCIAL PROTECTION Indicators for Zambia",
                    "description": "*FINANCIAL PROTECTION:*\n[Population with household expenditures on hea",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "913c96a9-48f9-4e4b-b11b-8ca019fdfd90",
                    "name": "Noncommunicable diseases and mental health Indicators for Zambia",
                    "description": "*Noncommunicable diseases and mental health:*\n[Number of deaths attrib",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ccb28805-c35a-4393-a7eb-6653c73cbde7",
                    "name": "Health workforce Indicators for Zambia",
                    "description": "*Health workforce:*\n[Medical doctors (per 10 000 population)](https://",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "12fdb6cb-5e00-45b5-a6b5-611847866440",
                    "name": "Neglected Tropical Diseases Indicators for Zambia",
                    "description": "*Neglected Tropical Diseases:*\n[Number of new leprosy cases](https://w",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ad2d521e-3a0e-451e-977c-bcbf125cec55",
                    "name": "QuickCharts Indicators for Zambia",
                    "description": "Cut down data for QuickCharts",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "19caff60-fd38-42b4-a17a-63abfaf0911e",
                    "name": "TOBACCO Indicators for Zambia",
                    "description": "*TOBACCO:*\n[Monitor](https://www.who.int/data/gho/indicator-metadata-r",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b078ed42-079b-4a7c-b699-9c7894b87d4f",
                    "name": "UHC Indicators for Zambia",
                    "description": "*UHC:*\n[Population in malaria-endemic areas who slept under an insecti",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "61a61d4a-a50c-4248-bf3d-15c726ced591",
                    "name": "ICD Indicators for Zambia",
                    "description": "*ICD:*\n[ICD-11 implementation progress level](https://web-prod.who.int",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "beea81a0-dea1-45a3-891c-36d48a2c8bde",
                    "name": "SEXUAL AND REPRODUCTIVE HEALTH Indicators for Zambia",
                    "description": "*SEXUAL AND REPRODUCTIVE HEALTH:*\n[Institutional Births (birth taken p",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9ed8a43b-d396-4a6f-948f-5bfe4c2fbba5",
                    "name": "Immunization Indicators for Zambia",
                    "description": "*Immunization:*\n[Proportion of vaccination cards seen (%)](https://www",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "3a70eae8-5f44-497c-92c4-dd173e702429",
                    "name": "NLIS Indicators for Zambia",
                    "description": "*NLIS:*\n[Subclinical vitamin A deficiency in preschool-age children (s",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
            ],
            "state": "active",
        }
        self.check_resources(update, results)

    def test_update_logic_2(
        self,
        configuration,
        new_dataset,
        dataset,
        new_resources_yaml,
        resources_yaml,
    ):
        self.add_new_dataset_resources(
            new_dataset, new_resources_yaml, exclude=[52]
        )
        self.add_dataset_resources(dataset, resources_yaml)
        dataset.old_data = new_dataset.data
        dataset.old_data["resources"] = new_dataset.resources
        results = dataset._dataset_merge_hdx_update(
            update_resources=True,
            match_resources_by_metadata=True,
            keys_to_delete=list(),
            remove_additional_resources=True,
            match_resource_order=False,
            create_default_views=False,
            hxl_update=False,
            test=True,
        )
        assert results["filter"] == ["-resources__53"]
        update = results["update"]
        del update["updated_by_script"]
        assert update == {
            "name": "who-data-for-zambia",
            "title": "Zambia - Health Indicators",
            "private": False,
            "notes": "Contains data from World Health Organization...",
            "dataset_source": "World Health Organization",
            "license_id": "hdx-other",
            "license_other": "CC BY-NC-SA 3.0 IGO",
            "methodology": "Registry",
            "maintainer": "35f7bb2c-4ab6-4796-8334-525b30a94c89",
            "owner_org": "c021f6be-3598-418e-8f7f-c7a799194dba",
            "data_update_frequency": "30",
            "subnational": "0",
            "dataset_date": "[1961-01-01T00:00:00 TO 2019-12-31T00:00:00]",
            "groups": [{"name": "zmb"}],
            "tags": [
                {
                    "name": "hxl",
                    "vocabulary_id": "4e61d464-4943-4e97-973a-84673c1aaa87",
                }
            ],
            "id": "3adc4bb0-faef-42ae-bd67-0ea08918a629",
            "resources": [
                {
                    "id": "1e7a68da-501a-444e-94f1-3606263e10c8",
                    "name": "All Health Indicators for Zambia",
                    "description": "See resource descriptions below for links to indicator metadata",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "40fbcfcc-e180-455d-9d65-a9f8d1f85643",
                    "name": "Mortality and global health estimates Indicators for Zambia",
                    "description": "*Mortality and global health estimates:*\n[Infant mortality rate (proba",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "018577c8-a397-4a4f-b13c-889c4990d5e2",
                    "name": "Sustainable development goals Indicators for Zambia",
                    "description": "*Sustainable development goals:*\n[Adolescent birth rate (per 1000 wome",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b42f3a3e-82a9-4fe6-9f0c-81d520ce7666",
                    "name": "Millennium Development Goals (MDGs) Indicators for Zambia",
                    "description": "*Millennium Development Goals (MDGs):*\n[Contraceptive prevalence (%)](",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b67738ca-812f-41b2-80e3-7c3340648398",
                    "name": "Health systems Indicators for Zambia",
                    "description": "*Health systems:*\n[Median availability of selected generic medicines (",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "a610cbb6-3bd2-4400-9aee-899dbe6b4a98",
                    "name": "Malaria Indicators for Zambia",
                    "description": "*Malaria:*\n[Children aged <5 years sleeping under insecticide-treated ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "de38982e-ef0d-4e5c-b0a3-16f933eaa808",
                    "name": "Tuberculosis Indicators for Zambia",
                    "description": "*Tuberculosis:*\n[Deaths due to tuberculosis among HIV-negative people ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "c330fc6e-624d-4d7f-bb46-f5ea67cf5d0a",
                    "name": "Child health Indicators for Zambia",
                    "description": "*Child health:*\n[Children aged <5 years stunted (%)](https://www.who.i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9903aea5-d193-4d9d-9018-e98213aef0f5",
                    "name": "Infectious diseases Indicators for Zambia",
                    "description": "*Infectious diseases:*\n[Cholera - number of reported cases](https://ww",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "fb24e4d1-9447-496d-9049-4712e4cd90c6",
                    "name": "World Health Statistics Indicators for Zambia",
                    "description": "*World Health Statistics:*\n[Literacy rate among adults aged >= 15 year",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "98e022a6-e054-41fd-af9a-e4c08ddcc5cd",
                    "name": "Health financing Indicators for Zambia",
                    "description": "*Health financing:*\n[Private prepaid plans as a percentage of private ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9a72db07-1bcc-4316-a427-1c208ebb5708",
                    "name": "Public health and environment Indicators for Zambia",
                    "description": "*Public health and environment:*\n[Population using solid fuels (%)](ht",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "873ead28-fff0-49d6-83c4-1ccda19cb0f7",
                    "name": "Substance use and mental health Indicators for Zambia",
                    "description": "*Substance use and mental health:*\n[Fines for violations](https://www.",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "c07999f6-6ab3-4f43-abb5-e3bb245edceb",
                    "name": "Tobacco Indicators for Zambia",
                    "description": "*Tobacco:*\n[Prevalence of smoking any tobacco product among persons ag",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "74e93f31-494c-4950-b891-6db42537e8b4",
                    "name": "Injuries and violence Indicators for Zambia",
                    "description": "*Injuries and violence:*\n[Income level](https://www.who.int/data/gho/i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ef319875-76fb-4ff4-bddb-02715a7638df",
                    "name": "HIV/AIDS and other STIs Indicators for Zambia",
                    "description": "*HIV/AIDS and other STIs:*\n[Prevalence of HIV among adults aged 15 to ",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1d4b9ac1-71ed-4ed3-a120-c4c6fea00a3d",
                    "name": "Nutrition Indicators for Zambia",
                    "description": "*Nutrition:*\n[Early initiation of breastfeeding (%)](https://www.who.i",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1815f97a-d93c-4e64-909d-39f288880c50",
                    "name": "Urban health Indicators for Zambia",
                    "description": "*Urban health:*\n[Percentage of the total population living in cities >",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "23af66cb-cffb-491f-a51b-c41584b83f3b",
                    "name": "Noncommunicable diseases Indicators for Zambia",
                    "description": "*Noncommunicable diseases:*\n[Prevalence of overweight among adults, BM",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2beb31ac-873d-47f4-9cfc-75cebd2303b5",
                    "name": "Noncommunicable diseases CCS Indicators for Zambia",
                    "description": "*Noncommunicable diseases CCS:*\n[Existence of operational policy/strat",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9a14881d-196a-44dc-bcf6-bda199b1137b",
                    "name": "Negelected tropical diseases Indicators for Zambia",
                    "description": "*Negelected tropical diseases:*\n[Number of new reported cases of Burul",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "79b0d05f-9180-492c-b614-687a9203bc1c",
                    "name": "Health Equity Monitor Indicators for Zambia",
                    "description": "*Health Equity Monitor:*\n[Antenatal care coverage - at least one visit",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "4b4defd7-ce45-46e3-86ce-15154be90d0c",
                    "name": "Infrastructure Indicators for Zambia",
                    "description": "*Infrastructure:*\n[Total density per 100 000 population: Health posts]",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "29ac1338-b7b5-429f-bb38-455a66e8f6c1",
                    "name": "Essential health technologies Indicators for Zambia",
                    "description": "*Essential health technologies:*\n[Availability of national standards o",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "63ba2693-fe73-4555-9c64-93710f7c7404",
                    "name": "Medical equipment Indicators for Zambia",
                    "description": "*Medical equipment:*\n[Total density per million population: Magnetic R",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2cdc3198-4ec7-4a25-86ee-8cdc80b670fc",
                    "name": "Demographic and socioeconomic statistics Indicators for Zambia",
                    "description": "*Demographic and socioeconomic statistics:*\n[Cellular subscribers (per",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1028087c-af76-4ab2-ac30-0babb4ef00fc",
                    "name": "Neglected tropical diseases Indicators for Zambia",
                    "description": "*Neglected tropical diseases:*\n[Status of yaws endemicity](https://www",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "0ec8b0b2-1e0d-4338-b22a-e984f8d07f0a",
                    "name": "International Health Regulations (2005) monitoring framework Indicators for Zambia",
                    "description": "*International Health Regulations (2005) monitoring framework:*\n[Legis",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "2fa6877b-5b6d-446f-9ccf-52e2478d86da",
                    "name": "Insecticide resistance Indicators for Zambia",
                    "description": "*Insecticide resistance:*\n[Number of insecticide classes to which resi",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "262fddcb-62eb-42f1-94c6-ed20e6b18650",
                    "name": "Universal Health Coverage Indicators for Zambia",
                    "description": "*Universal Health Coverage:*\n[Cataract surgical coverage of adults age",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5562627b-b7af-4d4d-a9ca-e4fcd996a180",
                    "name": "Global Observatory for eHealth (GOe) Indicators for Zambia",
                    "description": "*Global Observatory for eHealth (GOe):*\n[National universal health cov",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "53a1eeda-7a53-4b8c-87c8-c2ea276a52a3",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING : PREVENTION Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING : PREVENTION:*\n[Government uni",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "64c1aa0d-fa9a-4387-812e-fa7d00f382c2",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING: TREATMENT Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING: TREATMENT:*\n[Government unit/",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5b82a27c-a461-475b-867b-0afdc93132fd",
                    "name": "RSUD: GOVERNANCE, POLICY AND FINANCING: FINANCING Indicators for Zambia",
                    "description": "*RSUD: GOVERNANCE, POLICY AND FINANCING: FINANCING:*\n[Five-year change",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "f0f95f26-0701-49ee-a9ac-d1525572f6a5",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT SECTORS AND PROVIDERS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT SECTORS AND PROVID",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9b805d00-3de4-48f2-a93f-d859961e9c08",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT CAPACITY AND TREATMENT COVERAGE Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: TREATMENT CAPACITY AND TREAT",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9667ee27-97e5-4122-9dc6-e39df70eeb08",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: PHARMACOLOGICAL TREATMENT Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: PHARMACOLOGICAL TREATMENT:*\n",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "5e16b87e-5a06-4f7f-badb-3a2e44d64948",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: SCREENING AND BRIEF INTERVENTIONS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: SCREENING AND BRIEF INTERVEN",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "d79c3183-471f-4331-94d6-13474171be91",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: PREVENTION PROGRAMS AND PROVIDERS Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: PREVENTION PROGRAMS AND PROV",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "daa99f76-d84b-4c05-b5dc-c9e82fbcb859",
                    "name": "RSUD: SERVICE ORGANIZATION AND DELIVERY: SPECIAL PROGRAMMES AND SERVICES Indicators for Zambia",
                    "description": "*RSUD: SERVICE ORGANIZATION AND DELIVERY: SPECIAL PROGRAMMES AND SERVI",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ca1774e9-49c0-42a2-93c0-88fa32bb5adc",
                    "name": "RSUD: HUMAN RESOURCES Indicators for Zambia",
                    "description": "*RSUD: HUMAN RESOURCES:*\n[Health professionals providing treatment for",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "d244efc2-aa38-4197-b594-9bbd2b44bd9a",
                    "name": "RSUD: INFORMATION SYSTEMS Indicators for Zambia",
                    "description": "*RSUD: INFORMATION SYSTEMS:*\n[Epidemiological data collection for subs",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ee06d52b-5c15-4a0c-8d00-830215aeff03",
                    "name": "RSUD: YOUTH Indicators for Zambia",
                    "description": "*RSUD: YOUTH:*\n[Epidemiological data collection system for substance u",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "1ef02dd7-07cc-44f9-83ee-fc945f6d22ea",
                    "name": "FINANCIAL PROTECTION Indicators for Zambia",
                    "description": "*FINANCIAL PROTECTION:*\n[Population with household expenditures on hea",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "913c96a9-48f9-4e4b-b11b-8ca019fdfd90",
                    "name": "Noncommunicable diseases and mental health Indicators for Zambia",
                    "description": "*Noncommunicable diseases and mental health:*\n[Number of deaths attrib",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ccb28805-c35a-4393-a7eb-6653c73cbde7",
                    "name": "Health workforce Indicators for Zambia",
                    "description": "*Health workforce:*\n[Medical doctors (per 10 000 population)](https://",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "12fdb6cb-5e00-45b5-a6b5-611847866440",
                    "name": "Neglected Tropical Diseases Indicators for Zambia",
                    "description": "*Neglected Tropical Diseases:*\n[Number of new leprosy cases](https://w",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "ad2d521e-3a0e-451e-977c-bcbf125cec55",
                    "name": "QuickCharts Indicators for Zambia",
                    "description": "Cut down data for QuickCharts",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "19caff60-fd38-42b4-a17a-63abfaf0911e",
                    "name": "TOBACCO Indicators for Zambia",
                    "description": "*TOBACCO:*\n[Monitor](https://www.who.int/data/gho/indicator-metadata-r",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "b078ed42-079b-4a7c-b699-9c7894b87d4f",
                    "name": "UHC Indicators for Zambia",
                    "description": "*UHC:*\n[Population in malaria-endemic areas who slept under an insecti",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "61a61d4a-a50c-4248-bf3d-15c726ced591",
                    "name": "ICD Indicators for Zambia",
                    "description": "*ICD:*\n[ICD-11 implementation progress level](https://web-prod.who.int",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "beea81a0-dea1-45a3-891c-36d48a2c8bde",
                    "name": "SEXUAL AND REPRODUCTIVE HEALTH Indicators for Zambia",
                    "description": "*SEXUAL AND REPRODUCTIVE HEALTH:*\n[Institutional Births (birth taken p",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "id": "9ed8a43b-d396-4a6f-948f-5bfe4c2fbba5",
                    "name": "Immunization Indicators for Zambia",
                    "description": "*Immunization:*\n[Proportion of vaccination cards seen (%)](https://www",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
            ],
            "state": "active",
        }
        self.check_resources(update, results)

    def test_update_logic_3(
        self,
        configuration,
        new_dataset,
        dataset,
        new_resources_yaml,
        resources_yaml,
    ):
        self.add_new_dataset_resources(
            new_dataset, new_resources_yaml, include=[0, 3]
        )
        self.add_dataset_resources(dataset, resources_yaml, include=[0, 1, 2])
        dataset.old_data = new_dataset.data
        dataset.old_data["resources"] = new_dataset.resources
        results = dataset._dataset_merge_hdx_update(
            update_resources=True,
            match_resources_by_metadata=True,
            keys_to_delete=list(),
            remove_additional_resources=True,
            match_resource_order=False,
            create_default_views=False,
            hxl_update=False,
            test=True,
        )
        assert results["filter"] == list()
        update = results["update"]
        del update["updated_by_script"]
        assert update == {
            "name": "who-data-for-zambia",
            "title": "Zambia - Health Indicators",
            "private": False,
            "notes": "Contains data from World Health Organization...",
            "dataset_source": "World Health Organization",
            "license_id": "hdx-other",
            "license_other": "CC BY-NC-SA 3.0 IGO",
            "methodology": "Registry",
            "maintainer": "35f7bb2c-4ab6-4796-8334-525b30a94c89",
            "owner_org": "c021f6be-3598-418e-8f7f-c7a799194dba",
            "data_update_frequency": "30",
            "subnational": "0",
            "dataset_date": "[1961-01-01T00:00:00 TO 2019-12-31T00:00:00]",
            "groups": [{"name": "zmb"}],
            "tags": [
                {
                    "name": "hxl",
                    "vocabulary_id": "4e61d464-4943-4e97-973a-84673c1aaa87",
                }
            ],
            "id": "3adc4bb0-faef-42ae-bd67-0ea08918a629",
            "resources": [
                {
                    "id": "1e7a68da-501a-444e-94f1-3606263e10c8",
                    "name": "All Health Indicators for Zambia",
                    "description": "See resource descriptions below for links to indicator metadata",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
                {
                    "name": "Millennium Development Goals (MDGs) Indicators for Zambia",
                    "description": "*Millennium Development Goals (MDGs):*\n[Contraceptive prevalence (%)](",
                    "format": "csv",
                    "url_type": "upload",
                    "resource_type": "file.upload",
                    "url": "updated_by_file_upload_step",
                },
            ],
            "state": "active",
        }
        self.check_resources(update, results)



"""Simple Facade Tests"""
import sys

import pytest
from hdx.utilities.useragent import UserAgent

from hdx.api.configuration import ConfigurationError
from hdx.facades.infer_arguments import facade

from . import my_testfnia, testresult


class TestInferArguments:
    def test_facade(self, monkeypatch, hdx_config_yaml, project_config_yaml):
        UserAgent.clear_global()
        mydata = "hello"
        monkeypatch.setattr(
            sys,
            "argv",
            [
                "test",
                "--mydata",
                mydata,
                "--hdx-site",
                "prod",
                "--hdx-key",
                "123",
                "--user-agent",
                "test",
            ],
        )
        testresult.actual_result = None
        facade(my_testfnia)
        assert testresult.actual_result == mydata

        UserAgent.clear_global()
        monkeypatch.setattr(
            sys,
            "argv",
            [
                "test",
                "--mydata",
                mydata,
                "--hdx-site",
                "prod",
                "--user-agent",
                "test",
            ],
        )
        with pytest.raises(ConfigurationError):
            facade(my_testfnia)

        UserAgent.clear_global()
        monkeypatch.setattr(
            sys,
            "argv",
            [
                "test",
                "--mydata",
                mydata,
                "--hdx-site",
                "prod",
                "--hdx-key",
                "123",
                "--user-agent",
                "test",
                "--lala",
                "what",
            ],
        )
        with pytest.raises(SystemExit):
            facade(my_testfnia)
        UserAgent.clear_global()



"""Global fixtures"""
import smtplib
from os.path import join

import pytest

from hdx.api.configuration import Configuration
from hdx.api.locations import Locations


@pytest.fixture(scope="session")
def fixturesfolder():
    return join("tests", "fixtures")


@pytest.fixture(scope="session")
def configfolder(fixturesfolder):
    return join(fixturesfolder, "config")


@pytest.fixture(scope="session")
def hdx_config_yaml():
    return join("tests", "fixtures", "config", "hdx_config.yml")


@pytest.fixture(scope="session")
def project_config_yaml():
    return join("tests", "fixtures", "config", "project_configuration.yml")


@pytest.fixture(scope="session")
def locations():
    Locations.set_validlocations(
        [
            {
                "display_name": "Aruba",
                "id": "abw",
                "name": "abw",
                "title": "Aruba",
            },
            {
                "display_name": "Afghanistan",
                "id": "afg",
                "name": "afg",
                "title": "Afghanistan",
            },
            {
                "display_name": "Angola",
                "id": "ago",
                "name": "ago",
                "title": "Angola",
            },
            {
                "display_name": "Anguilla",
                "id": "aia",
                "name": "aia",
                "title": "Anguilla",
            },
            {
                "display_name": "Albania",
                "id": "alb",
                "name": "alb",
                "title": "Albania",
            },
            {
                "display_name": "Andorra",
                "id": "and",
                "name": "and",
                "title": "Andorra",
            },
            {
                "display_name": "Netherlands Antilles",
                "id": "ant",
                "name": "ant",
                "title": "Netherlands Antilles",
            },
            {
                "display_name": "United Arab Emirates",
                "id": "are",
                "name": "are",
                "title": "United Arab Emirates",
            },
            {
                "display_name": "Argentina",
                "id": "arg",
                "name": "arg",
                "title": "Argentina",
            },
            {
                "display_name": "Armenia",
                "id": "arm",
                "name": "arm",
                "title": "Armenia",
            },
            {
                "display_name": "American Samoa",
                "id": "asm",
                "name": "asm",
                "title": "American Samoa",
            },
            {
                "display_name": "Antartica",
                "id": "ata",
                "name": "ata",
                "title": "Antartica",
            },
            {
                "display_name": "French Southern Territories",
                "id": "atf",
                "name": "atf",
                "title": "French Southern Territories",
            },
            {
                "display_name": "Antigua And Barbuda",
                "id": "atg",
                "name": "atg",
                "title": "Antigua And Barbuda",
            },
            {
                "display_name": "Australia",
                "id": "aus",
                "name": "aus",
                "title": "Australia",
            },
            {
                "display_name": "Austria",
                "id": "aut",
                "name": "aut",
                "title": "Austria",
            },
            {
                "display_name": "Azerbajan",
                "id": "aze",
                "name": "aze",
                "title": "Azerbajan",
            },
            {
                "display_name": "Burundi",
                "id": "bdi",
                "name": "bdi",
                "title": "Burundi",
            },
            {
                "display_name": "Belgium",
                "id": "bel",
                "name": "bel",
                "title": "Belgium",
            },
            {
                "display_name": "Benin",
                "id": "ben",
                "name": "ben",
                "title": "Benin",
            },
            {
                "display_name": "Burkina Faso",
                "id": "bfa",
                "name": "bfa",
                "title": "Burkina Faso",
            },
            {
                "display_name": "Bangladesh",
                "id": "bgd",
                "name": "bgd",
                "title": "Bangladesh",
            },
            {
                "display_name": "Bulgaria",
                "id": "bgr",
                "name": "bgr",
                "title": "Bulgaria",
            },
            {
                "display_name": "Bahrain",
                "id": "bhr",
                "name": "bhr",
                "title": "Bahrain",
            },
            {
                "display_name": "Bahamas",
                "id": "bhs",
                "name": "bhs",
                "title": "Bahamas",
            },
            {
                "display_name": "Bosnia And Herzegowina",
                "id": "bih",
                "name": "bih",
                "title": "Bosnia And Herzegowina",
            },
            {
                "display_name": "Belarus",
                "id": "blr",
                "name": "blr",
                "title": "Belarus",
            },
            {
                "display_name": "Belize",
                "id": "blz",
                "name": "blz",
                "title": "Belize",
            },
            {
                "display_name": "Bermuda",
                "id": "bmu",
                "name": "bmu",
                "title": "Bermuda",
            },
            {
                "display_name": "Bolivia (Plurinational State of)",
                "id": "bol",
                "name": "bol",
                "title": "Bolivia (Plurinational State of)",
            },
            {
                "display_name": "Brazil",
                "id": "bra",
                "name": "bra",
                "title": "Brazil",
            },
            {
                "display_name": "Barbados",
                "id": "brb",
                "name": "brb",
                "title": "Barbados",
            },
            {
                "display_name": "Brunei Darussalam",
                "id": "brn",
                "name": "brn",
                "title": "Brunei Darussalam",
            },
            {
                "display_name": "Bhutan",
                "id": "btn",
                "name": "btn",
                "title": "Bhutan",
            },
            {
                "display_name": "Bouvet Island",
                "id": "bvt",
                "name": "bvt",
                "title": "Bouvet Island",
            },
            {
                "display_name": "Botswana",
                "id": "bwa",
                "name": "bwa",
                "title": "Botswana",
            },
            {
                "display_name": "Central African Republic",
                "id": "caf",
                "name": "caf",
                "title": "Central African Republic",
            },
            {
                "display_name": "Canada",
                "id": "can",
                "name": "can",
                "title": "Canada",
            },
            {
                "display_name": "Cocos (Keeling) Islands",
                "id": "cck",
                "name": "cck",
                "title": "Cocos (Keeling) Islands",
            },
            {
                "display_name": "Switzerland",
                "id": "che",
                "name": "che",
                "title": "Switzerland",
            },
            {
                "display_name": "Chile",
                "id": "chl",
                "name": "chl",
                "title": "Chile",
            },
            {
                "display_name": "China",
                "id": "chn",
                "name": "chn",
                "title": "China",
            },
            {
                "display_name": "Côte d'Ivoire",
                "id": "civ",
                "name": "civ",
                "title": "Côte d'Ivoire",
            },
            {
                "display_name": "Cameroon",
                "id": "cmr",
                "name": "cmr",
                "title": "Cameroon",
            },
            {
                "display_name": "Democratic Republic of the Congo",
                "id": "cod",
                "name": "cod",
                "title": "Democratic Republic of the Congo",
            },
            {
                "display_name": "Congo",
                "id": "cog",
                "name": "cog",
                "title": "Congo",
            },
            {
                "display_name": "Cook Islands",
                "id": "cok",
                "name": "cok",
                "title": "Cook Islands",
            },
            {
                "display_name": "Colombia",
                "id": "col",
                "name": "col",
                "title": "Colombia",
            },
            {
                "display_name": "Comoros",
                "id": "com",
                "name": "com",
                "title": "Comoros",
            },
            {
                "display_name": "Cape Verde",
                "id": "cpv",
                "name": "cpv",
                "title": "Cape Verde",
            },
            {
                "display_name": "Costa Rica",
                "id": "cri",
                "name": "cri",
                "title": "Costa Rica",
            },
            {
                "display_name": "Cuba",
                "id": "cub",
                "name": "cub",
                "title": "Cuba",
            },
            {
                "display_name": "Christmas Island",
                "id": "cxr",
                "name": "cxr",
                "title": "Christmas Island",
            },
            {
                "display_name": "Cayman Islands",
                "id": "cym",
                "name": "cym",
                "title": "Cayman Islands",
            },
            {
                "display_name": "Cyprus",
                "id": "cyp",
                "name": "cyp",
                "title": "Cyprus",
            },
            {
                "display_name": "Czech Republic",
                "id": "cze",
                "name": "cze",
                "title": "Czech Republic",
            },
            {
                "display_name": "Germany",
                "id": "deu",
                "name": "deu",
                "title": "Germany",
            },
            {
                "display_name": "Djibouti",
                "id": "dji",
                "name": "dji",
                "title": "Djibouti",
            },
            {
                "display_name": "Dominica",
                "id": "dma",
                "name": "dma",
                "title": "Dominica",
            },
            {
                "display_name": "Denmark",
                "id": "dnk",
                "name": "dnk",
                "title": "Denmark",
            },
            {
                "display_name": "Dominican Republic",
                "id": "dom",
                "name": "dom",
                "title": "Dominican Republic",
            },
            {
                "display_name": "Algeria",
                "id": "dza",
                "name": "dza",
                "title": "Algeria",
            },
            {
                "display_name": "Ecuador",
                "id": "ecu",
                "name": "ecu",
                "title": "Ecuador",
            },
            {
                "display_name": "Egypt",
                "id": "egy",
                "name": "egy",
                "title": "Egypt",
            },
            {
                "display_name": "Eritrea",
                "id": "eri",
                "name": "eri",
                "title": "Eritrea",
            },
            {
                "display_name": "Western Sahara",
                "id": "esh",
                "name": "esh",
                "title": "Western Sahara",
            },
            {
                "display_name": "Spain",
                "id": "esp",
                "name": "esp",
                "title": "Spain",
            },
            {
                "display_name": "Estonia",
                "id": "est",
                "name": "est",
                "title": "Estonia",
            },
            {
                "display_name": "Ethiopia",
                "id": "eth",
                "name": "eth",
                "title": "Ethiopia",
            },
            {
                "display_name": "Finland",
                "id": "fin",
                "name": "fin",
                "title": "Finland",
            },
            {
                "display_name": "Fiji",
                "id": "fji",
                "name": "fji",
                "title": "Fiji",
            },
            {
                "display_name": "Falkland Islands (Malvinas)",
                "id": "flk",
                "name": "flk",
                "title": "Falkland Islands (Malvinas)",
            },
            {
                "display_name": "France",
                "id": "fra",
                "name": "fra",
                "title": "France",
            },
            {
                "display_name": "Faroe Islands",
                "id": "fro",
                "name": "fro",
                "title": "Faroe Islands",
            },
            {
                "display_name": "Micronesia, Federated States Of",
                "id": "fsm",
                "name": "fsm",
                "title": "Micronesia, Federated States Of",
            },
            {
                "display_name": "France, Metropolitan",
                "id": "fxx",
                "name": "fxx",
                "title": "France, Metropolitan",
            },
            {
                "display_name": "Gabon",
                "id": "gab",
                "name": "gab",
                "title": "Gabon",
            },
            {
                "display_name": "United Kingdom",
                "id": "gbr",
                "name": "gbr",
                "title": "United Kingdom",
            },
            {
                "display_name": "Georgia",
                "id": "geo",
                "name": "geo",
                "title": "Georgia",
            },
            {
                "display_name": "Ghana",
                "id": "gha",
                "name": "gha",
                "title": "Ghana",
            },
            {
                "display_name": "Gibraltar",
                "id": "gib",
                "name": "gib",
                "title": "Gibraltar",
            },
            {
                "display_name": "Guinea",
                "id": "gin",
                "name": "gin",
                "title": "Guinea",
            },
            {
                "display_name": "Guadeloupe",
                "id": "glp",
                "name": "glp",
                "title": "Guadeloupe",
            },
            {
                "display_name": "Gambia",
                "id": "gmb",
                "name": "gmb",
                "title": "Gambia",
            },
            {
                "display_name": "Guinea-Bissau",
                "id": "gnb",
                "name": "gnb",
                "title": "Guinea-Bissau",
            },
            {
                "display_name": "Equatorial Guinea",
                "id": "gnq",
                "name": "gnq",
                "title": "Equatorial Guinea",
            },
            {
                "display_name": "Greece",
                "id": "grc",
                "name": "grc",
                "title": "Greece",
            },
            {
                "display_name": "Grenada",
                "id": "grd",
                "name": "grd",
                "title": "Grenada",
            },
            {
                "display_name": "Greenland",
                "id": "grl",
                "name": "grl",
                "title": "Greenland",
            },
            {
                "display_name": "Guatemala",
                "id": "gtm",
                "name": "gtm",
                "title": "Guatemala",
            },
            {
                "display_name": "French Guiana",
                "id": "guf",
                "name": "guf",
                "title": "French Guiana",
            },
            {
                "display_name": "Guam",
                "id": "gum",
                "name": "gum",
                "title": "Guam",
            },
            {
                "display_name": "Guyana",
                "id": "guy",
                "name": "guy",
                "title": "Guyana",
            },
            {
                "display_name": "Hong Kong",
                "id": "hkg",
                "name": "hkg",
                "title": "Hong Kong",
            },
            {
                "display_name": "Heard And McDonald Islands",
                "id": "hmd",
                "name": "hmd",
                "title": "Heard And McDonald Islands",
            },
            {
                "display_name": "Honduras",
                "id": "hnd",
                "name": "hnd",
                "title": "Honduras",
            },
            {
                "display_name": "Croatia",
                "id": "hrv",
                "name": "hrv",
                "title": "Croatia",
            },
            {
                "display_name": "Haiti",
                "id": "hti",
                "name": "hti",
                "title": "Haiti",
            },
            {
                "display_name": "Hungary",
                "id": "hun",
                "name": "hun",
                "title": "Hungary",
            },
            {
                "display_name": "Indonesia",
                "id": "idn",
                "name": "idn",
                "title": "Indonesia",
            },
            {
                "display_name": "India",
                "id": "ind",
                "name": "ind",
                "title": "India",
            },
            {
                "display_name": "British Indian Ocean Territory",
                "id": "iot",
                "name": "iot",
                "title": "British Indian Ocean Territory",
            },
            {
                "display_name": "Ireland",
                "id": "irl",
                "name": "irl",
                "title": "Ireland",
            },
            {
                "display_name": "Iran (Islamic Republic Of)",
                "id": "irn",
                "name": "irn",
                "title": "Iran (Islamic Republic Of)",
            },
            {
                "display_name": "Iraq",
                "id": "irq",
                "name": "irq",
                "title": "Iraq",
            },
            {
                "display_name": "Iceland",
                "id": "isl",
                "name": "isl",
                "title": "Iceland",
            },
            {
                "display_name": "Israel",
                "id": "isr",
                "name": "isr",
                "title": "Israel",
            },
            {
                "display_name": "Italy",
                "id": "ita",
                "name": "ita",
                "title": "Italy",
            },
            {
                "display_name": "Jamaica",
                "id": "jam",
                "name": "jam",
                "title": "Jamaica",
            },
            {
                "display_name": "Jordan",
                "id": "jor",
                "name": "jor",
                "title": "Jordan",
            },
            {
                "display_name": "Japan",
                "id": "jpn",
                "name": "jpn",
                "title": "Japan",
            },
            {
                "display_name": "Kazakhstan",
                "id": "kaz",
                "name": "kaz",
                "title": "Kazakhstan",
            },
            {
                "display_name": "Kenya",
                "id": "ken",
                "name": "ken",
                "title": "Kenya",
            },
            {
                "display_name": "Kyrgyzstan",
                "id": "kgz",
                "name": "kgz",
                "title": "Kyrgyzstan",
            },
            {
                "display_name": "Cambodia",
                "id": "khm",
                "name": "khm",
                "title": "Cambodia",
            },
            {
                "display_name": "Kiribati",
                "id": "kir",
                "name": "kir",
                "title": "Kiribati",
            },
            {
                "display_name": "Saint Kitts And Nevis",
                "id": "kna",
                "name": "kna",
                "title": "Saint Kitts And Nevis",
            },
            {
                "display_name": "Korea, Republic Of",
                "id": "kor",
                "name": "kor",
                "title": "Korea, Republic Of",
            },
            {
                "display_name": "Kuwait",
                "id": "kwt",
                "name": "kwt",
                "title": "Kuwait",
            },
            {
                "display_name": "Lao People's Democratic Republic",
                "id": "lao",
                "name": "lao",
                "title": "Lao People's Democratic Republic",
            },
            {
                "display_name": "Lebanon",
                "id": "lbn",
                "name": "lbn",
                "title": "Lebanon",
            },
            {
                "display_name": "Liberia",
                "id": "lbr",
                "name": "lbr",
                "title": "Liberia",
            },
            {
                "display_name": "Libya",
                "id": "lby",
                "name": "lby",
                "title": "Libya",
            },
            {
                "display_name": "Saint Lucia",
                "id": "lca",
                "name": "lca",
                "title": "Saint Lucia",
            },
            {
                "display_name": "Liechtenstein",
                "id": "lie",
                "name": "lie",
                "title": "Liechtenstein",
            },
            {
                "display_name": "Sri Lanka",
                "id": "lka",
                "name": "lka",
                "title": "Sri Lanka",
            },
            {
                "display_name": "Lesotho",
                "id": "lso",
                "name": "lso",
                "title": "Lesotho",
            },
            {
                "display_name": "Lithuania",
                "id": "ltu",
                "name": "ltu",
                "title": "Lithuania",
            },
            {
                "display_name": "Luxemburg",
                "id": "lux",
                "name": "lux",
                "title": "Luxemburg",
            },
            {
                "display_name": "Latvia",
                "id": "lva",
                "name": "lva",
                "title": "Latvia",
            },
            {
                "display_name": "Macau",
                "id": "mac",
                "name": "mac",
                "title": "Macau",
            },
            {
                "display_name": "Morocco",
                "id": "mar",
                "name": "mar",
                "title": "Morocco",
            },
            {
                "display_name": "Monaco",
                "id": "mco",
                "name": "mco",
                "title": "Monaco",
            },
            {
                "display_name": "Moldova, Republic Of",
                "id": "mda",
                "name": "mda",
                "title": "Moldova, Republic Of",
            },
            {
                "display_name": "Madagascar",
                "id": "mdg",
                "name": "mdg",
                "title": "Madagascar",
            },
            {
                "display_name": "Maldives",
                "id": "mdv",
                "name": "mdv",
                "title": "Maldives",
            },
            {
                "display_name": "Mexico",
                "id": "mex",
                "name": "mex",
                "title": "Mexico",
            },
            {
                "display_name": "Marshall Islands",
                "id": "mhl",
                "name": "mhl",
                "title": "Marshall Islands",
            },
            {
                "display_name": "Macedonia",
                "id": "mkd",
                "name": "mkd",
                "title": "Macedonia",
            },
            {
                "display_name": "Mali",
                "id": "mli",
                "name": "mli",
                "title": "Mali",
            },
            {
                "display_name": "Malta",
                "id": "mlt",
                "name": "mlt",
                "title": "Malta",
            },
            {
                "display_name": "Myanmar",
                "id": "mmr",
                "name": "mmr",
                "title": "Myanmar",
            },
            {
                "display_name": "Montenegro",
                "id": "mne",
                "name": "mne",
                "title": "Montenegro",
            },
            {
                "display_name": "Mongolia",
                "id": "mng",
                "name": "mng",
                "title": "Mongolia",
            },
            {
                "display_name": "Northern Mariana Islands",
                "id": "mnp",
                "name": "mnp",
                "title": "Northern Mariana Islands",
            },
            {
                "display_name": "Mozambique",
                "id": "moz",
                "name": "moz",
                "title": "Mozambique",
            },
            {
                "display_name": "Mauritania",
                "id": "mrt",
                "name": "mrt",
                "title": "Mauritania",
            },
            {
                "display_name": "Montserrat",
                "id": "msr",
                "name": "msr",
                "title": "Montserrat",
            },
            {
                "display_name": "Martinique",
                "id": "mtq",
                "name": "mtq",
                "title": "Martinique",
            },
            {
                "display_name": "Mauritius",
                "id": "mus",
                "name": "mus",
                "title": "Mauritius",
            },
            {
                "display_name": "Malawi",
                "id": "mwi",
                "name": "mwi",
                "title": "Malawi",
            },
            {
                "display_name": "Malaysia",
                "id": "mys",
                "name": "mys",
                "title": "Malaysia",
            },
            {
                "display_name": "Mayotte",
                "id": "myt",
                "name": "myt",
                "title": "Mayotte",
            },
            {
                "display_name": "Namibia",
                "id": "nam",
                "name": "nam",
                "title": "Namibia",
            },
            {
                "display_name": "New Caledonia",
                "id": "ncl",
                "name": "ncl",
                "title": "New Caledonia",
            },
            {
                "display_name": "Nepal Earthquake",
                "id": "fba94eb6-a73b-407d-8acd-c16e61e4f0dc",
                "name": "nepal-earthquake",
                "title": "Nepal Earthquake",
            },
            {
                "display_name": "Niger",
                "id": "ner",
                "name": "ner",
                "title": "Niger",
            },
            {
                "display_name": "Norfolk Island",
                "id": "nfk",
                "name": "nfk",
                "title": "Norfolk Island",
            },
            {
                "display_name": "Nigeria",
                "id": "nga",
                "name": "nga",
                "title": "Nigeria",
            },
            {
                "display_name": "Nicaragua",
                "id": "nic",
                "name": "nic",
                "title": "Nicaragua",
            },
            {
                "display_name": "Niue",
                "id": "niu",
                "name": "niu",
                "title": "Niue",
            },
            {
                "display_name": "Netherlands",
                "id": "nld",
                "name": "nld",
                "title": "Netherlands",
            },
            {
                "display_name": "Norway",
                "id": "nor",
                "name": "nor",
                "title": "Norway",
            },
            {
                "display_name": "Nepal",
                "id": "npl",
                "name": "npl",
                "title": "Nepal",
            },
            {
                "display_name": "Nauru",
                "id": "nru",
                "name": "nru",
                "title": "Nauru",
            },
            {
                "display_name": "New Zealand",
                "id": "nzl",
                "name": "nzl",
                "title": "New Zealand",
            },
            {
                "display_name": "Oman",
                "id": "omn",
                "name": "omn",
                "title": "Oman",
            },
            {
                "display_name": "Pakistan",
                "id": "pak",
                "name": "pak",
                "title": "Pakistan",
            },
            {
                "display_name": "Panama",
                "id": "pan",
                "name": "pan",
                "title": "Panama",
            },
            {
                "display_name": "Pitcairn",
                "id": "pcn",
                "name": "pcn",
                "title": "Pitcairn",
            },
            {
                "display_name": "Peru",
                "id": "per",
                "name": "per",
                "title": "Peru",
            },
            {
                "display_name": "Philippines",
                "id": "phl",
                "name": "phl",
                "title": "Philippines",
            },
            {
                "display_name": "Palau",
                "id": "plw",
                "name": "plw",
                "title": "Palau",
            },
            {
                "display_name": "Papua New Guinea",
                "id": "png",
                "name": "png",
                "title": "Papua New Guinea",
            },
            {
                "display_name": "Poland",
                "id": "pol",
                "name": "pol",
                "title": "Poland",
            },
            {
                "display_name": "Puerto Rico",
                "id": "pri",
                "name": "pri",
                "title": "Puerto Rico",
            },
            {
                "display_name": "Democratic People's Republic of Korea",
                "id": "prk",
                "name": "prk",
                "title": "Democratic People's Republic of Korea",
            },
            {
                "display_name": "Portugal",
                "id": "prt",
                "name": "prt",
                "title": "Portugal",
            },
            {
                "display_name": "Paraguay",
                "id": "pry",
                "name": "pry",
                "title": "Paraguay",
            },
            {
                "display_name": "State of Palestine",
                "id": "pse",
                "name": "pse",
                "title": "State of Palestine",
            },
            {
                "display_name": "French Polynesia",
                "id": "pyf",
                "name": "pyf",
                "title": "French Polynesia",
            },
            {
                "display_name": "Qatar",
                "id": "qat",
                "name": "qat",
                "title": "Qatar",
            },
            {
                "display_name": "Reunion",
                "id": "reu",
                "name": "reu",
                "title": "Reunion",
            },
            {
                "display_name": "Romania",
                "id": "c92bd69f-54c0-4d02-ad30-d33fd1cd1393",
                "name": "rou",
                "title": "Romania",
            },
            {
                "display_name": "Russia",
                "id": "rus",
                "name": "rus",
                "title": "Russia",
            },
            {
                "display_name": "Rwanda",
                "id": "rwa",
                "name": "rwa",
                "title": "Rwanda",
            },
            {
                "display_name": "Saudi Arabia",
                "id": "sau",
                "name": "sau",
                "title": "Saudi Arabia",
            },
            {
                "display_name": "Sudan",
                "id": "sdn",
                "name": "sdn",
                "title": "Sudan",
            },
            {
                "display_name": "Senegal",
                "id": "sen",
                "name": "sen",
                "title": "Senegal",
            },
            {
                "display_name": "Singapore",
                "id": "sgp",
                "name": "sgp",
                "title": "Singapore",
            },
            {
                "display_name": "South Georgia And South S.S.",
                "id": "sgs",
                "name": "sgs",
                "title": "South Georgia And South S.S.",
            },
            {
                "display_name": "St. Helena",
                "id": "shn",
                "name": "shn",
                "title": "St. Helena",
            },
            {
                "display_name": "Svalbard And Jan Mayen Islands",
                "id": "sjm",
                "name": "sjm",
                "title": "Svalbard And Jan Mayen Islands",
            },
            {
                "display_name": "Solomon Islands",
                "id": "slb",
                "name": "slb",
                "title": "Solomon Islands",
            },
            {
                "display_name": "Sierra Leone",
                "id": "sle",
                "name": "sle",
                "title": "Sierra Leone",
            },
            {
                "display_name": "El Salvador",
                "id": "slv",
                "name": "slv",
                "title": "El Salvador",
            },
            {
                "display_name": "San Marino",
                "id": "smr",
                "name": "smr",
                "title": "San Marino",
            },
            {
                "display_name": "Somalia",
                "id": "som",
                "name": "som",
                "title": "Somalia",
            },
            {
                "display_name": "St. Pierre And Miquelon",
                "id": "spm",
                "name": "spm",
                "title": "St. Pierre And Miquelon",
            },
            {
                "display_name": "Serbia",
                "id": "srb",
                "name": "srb",
                "title": "Serbia",
            },
            {
                "display_name": "South Sudan",
                "id": "ssd",
                "name": "ssd",
                "title": "South Sudan",
            },
            {
                "display_name": "Sao Tome And Principe",
                "id": "stp",
                "name": "stp",
                "title": "Sao Tome And Principe",
            },
            {
                "display_name": "Suriname",
                "id": "sur",
                "name": "sur",
                "title": "Suriname",
            },
            {
                "display_name": "Slovakia (Slovak Republic)",
                "id": "svk",
                "name": "svk",
                "title": "Slovakia (Slovak Republic)",
            },
            {
                "display_name": "Slovenia",
                "id": "svn",
                "name": "svn",
                "title": "Slovenia",
            },
            {
                "display_name": "Sweden",
                "id": "swe",
                "name": "swe",
                "title": "Sweden",
            },
            {
                "display_name": "Swaziland",
                "id": "swz",
                "name": "swz",
                "title": "Swaziland",
            },
            {
                "display_name": "Seychelles",
                "id": "syc",
                "name": "syc",
                "title": "Seychelles",
            },
            {
                "display_name": "Syrian Arab Republic",
                "id": "syr",
                "name": "syr",
                "title": "Syrian Arab Republic",
            },
            {
                "display_name": "Turks And Caicos Islands",
                "id": "tca",
                "name": "tca",
                "title": "Turks And Caicos Islands",
            },
            {
                "display_name": "Chad",
                "id": "tcd",
                "name": "tcd",
                "title": "Chad",
            },
            {
                "display_name": "Togo",
                "id": "tgo",
                "name": "tgo",
                "title": "Togo",
            },
            {
                "display_name": "Thailand",
                "id": "tha",
                "name": "tha",
                "title": "Thailand",
            },
            {
                "display_name": "Tajikistan",
                "id": "tjk",
                "name": "tjk",
                "title": "Tajikistan",
            },
            {
                "display_name": "Tokelau",
                "id": "tkl",
                "name": "tkl",
                "title": "Tokelau",
            },
            {
                "display_name": "Turkmenistan",
                "id": "tkm",
                "name": "tkm",
                "title": "Turkmenistan",
            },
            {
                "display_name": "East Timor",
                "id": "86b999e4-6981-401e-b57d-e15ad5a9ec86",
                "name": "tls",
                "title": "East Timor",
            },
            {
                "display_name": "Tonga",
                "id": "ton",
                "name": "ton",
                "title": "Tonga",
            },
            {
                "display_name": "Trinidad And Tobago",
                "id": "tto",
                "name": "tto",
                "title": "Trinidad And Tobago",
            },
            {
                "display_name": "Tunisia",
                "id": "tun",
                "name": "tun",
                "title": "Tunisia",
            },
            {
                "display_name": "Turkey",
                "id": "tur",
                "name": "tur",
                "title": "Turkey",
            },
            {
                "display_name": "Tuvalu",
                "id": "tuv",
                "name": "tuv",
                "title": "Tuvalu",
            },
            {
                "display_name": "Taiwan, Province Of China",
                "id": "twn",
                "name": "twn",
                "title": "Taiwan, Province Of China",
            },
            {
                "display_name": "United Republic of Tanzania",
                "id": "tza",
                "name": "tza",
                "title": "United Republic of Tanzania",
            },
            {
                "display_name": "Uganda",
                "id": "uga",
                "name": "uga",
                "title": "Uganda",
            },
            {
                "display_name": "Ukraine",
                "id": "ukr",
                "name": "ukr",
                "title": "Ukraine",
            },
            {
                "display_name": "U.S. Minor Islands",
                "id": "umi",
                "name": "umi",
                "title": "U.S. Minor Islands",
            },
            {
                "display_name": "Uruguay",
                "id": "ury",
                "name": "ury",
                "title": "Uruguay",
            },
            {
                "display_name": "United States",
                "id": "usa",
                "name": "usa",
                "title": "United States",
            },
            {
                "display_name": "Uzbekistan",
                "id": "uzb",
                "name": "uzb",
                "title": "Uzbekistan",
            },
            {
                "display_name": "Holy See (Vatican City State)",
                "id": "vat",
                "name": "vat",
                "title": "Holy See (Vatican City State)",
            },
            {
                "display_name": "Saint Vincent And The Grenadines",
                "id": "vct",
                "name": "vct",
                "title": "Saint Vincent And The Grenadines",
            },
            {
                "display_name": "Venezuela",
                "id": "ven",
                "name": "ven",
                "title": "Venezuela",
            },
            {
                "display_name": "Virgin Islands (British)",
                "id": "vgb",
                "name": "vgb",
                "title": "Virgin Islands (British)",
            },
            {
                "display_name": "Virgin Islands (U.S.)",
                "id": "vir",
                "name": "vir",
                "title": "Virgin Islands (U.S.)",
            },
            {
                "display_name": "Viet Nam",
                "id": "vnm",
                "name": "vnm",
                "title": "Viet Nam",
            },
            {
                "display_name": "Vanuatu",
                "id": "vut",
                "name": "vut",
                "title": "Vanuatu",
            },
            {
                "display_name": "Wallis And Futuna Islands",
                "id": "wlf",
                "name": "wlf",
                "title": "Wallis And Futuna Islands",
            },
            {
                "display_name": "World",
                "id": "world",
                "name": "world",
                "title": "World",
            },
            {
                "display_name": "Samoa",
                "id": "wsm",
                "name": "wsm",
                "title": "Samoa",
            },
            {
                "display_name": "Yemen",
                "id": "yem",
                "name": "yem",
                "title": "Yemen",
            },
            {
                "display_name": "South Africa",
                "id": "zaf",
                "name": "zaf",
                "title": "South Africa",
            },
            {
                "display_name": "Zambia",
                "id": "zmb",
                "name": "zmb",
                "title": "Zambia",
            },
            {
                "display_name": "Zimbabwe",
                "id": "zwe",
                "name": "zwe",
                "title": "Zimbabwe",
            },
        ]
    )


@pytest.fixture(scope="function")
def configuration(hdx_config_yaml, project_config_yaml):
    Configuration._create(
        user_agent="test",
        hdx_config_yaml=hdx_config_yaml,
        project_config_yaml=project_config_yaml,
    )


@pytest.fixture(scope="function")
def mocksmtp(monkeypatch):
    class MockSMTPBase:
        type = None

        def __init__(self, **kwargs):
            self.initargs = kwargs

        def login(self, username, password):
            self.username = username
            self.password = password

        def sendmail(self, sender, recipients, msg, **kwargs):
            self.sender = sender
            self.recipients = recipients
            self.msg = msg
            self.send_args = kwargs

        @staticmethod
        def quit():
            pass

    class MockSMTPSSL(MockSMTPBase):
        type = "smtpssl"

    class MockLMTP(MockSMTPBase):
        type = "lmtp"

    class MockSMTP(MockSMTPBase):
        type = "smtp"

    monkeypatch.setattr(smtplib, "SMTP_SSL", MockSMTPSSL)
    monkeypatch.setattr(smtplib, "LMTP", MockLMTP)
    monkeypatch.setattr(smtplib, "SMTP", MockSMTP)



"""Helper to the Dataset class for handling matching resources.
"""
import collections
from typing import TYPE_CHECKING, List, Optional, Tuple

if TYPE_CHECKING:
    from hdx.data.resource import Resource


class ResourceMatcher:
    @staticmethod
    def match_resource_list(
        resources1: List["Resource"],
        resource2: "Resource",
    ) -> Optional[int]:
        """Helper method to find the index of a resource that matches a given resource

        Args:
            resources1 (List[Resource]): List of resources
            resource2 (Resource): Resource to match with list

        Returns:
            Optional[int]: Index of resource that matches in list or None
        """
        id2 = resource2.get("id")
        grouping2 = resource2.get("grouping")
        index1_match = None
        if id2 is not None:
            ids1 = [(x.get("id"), x.get("grouping")) for x in resources1]
            id2 = (resource2.get("id"), grouping2)

            if id2 is not None:
                try:
                    return ids1.index(id2)
                except ValueError:
                    pass
        names1 = [(x["name"], x.get("grouping")) for x in resources1]
        name2 = (resource2["name"], grouping2)
        formats1 = [x["format"].lower() for x in resources1]
        format2 = resource2["format"].lower()

        dupnames = {
            item
            for item, count in collections.Counter(names1).items()
            if count > 1
        }
        for i, name1 in enumerate(names1):
            if name1 != name2:
                continue
            if name1 in dupnames:
                if formats1[i] != format2:
                    continue
            index1_match = i
        return index1_match

    @staticmethod
    def match_resource_lists(
        resources1: List["Resource"],
        resources2: List["Resource"],
    ) -> Tuple[List, List, List, List]:
        """Helper method to match two lists of resources returning the indices that match in two lists and
        that don't match in two more lists

        Args:
            resources1 (List[Resource]): List of resources
            resources2 (List[Resource]): List of resources to match with first list

        Returns:
            Tuple[List, List, List, List]: Returns indices that match (2 lists) and that don't match (2 lists)
        """
        ids1 = [x.get("id") for x in resources1]
        groups1 = [x.get("grouping") for x in resources1]
        names1 = [x["name"] for x in resources1]
        formats1 = [x["format"].lower() for x in resources1]
        ids2 = [x.get("id") for x in resources2]
        groups2 = [x.get("grouping") for x in resources2]
        names2 = [x["name"] for x in resources2]
        formats2 = [x["format"].lower() for x in resources2]
        index1_matches = list()
        index2_matches = list()
        for i, id1 in enumerate(ids1):
            if id1 is None:
                continue
            for j, id2 in enumerate(ids2):
                if id2 is None:
                    continue
                if id1 == id2:
                    if groups1[i] == groups2[j]:
                        index1_matches.append(i)
                        index2_matches.append(j)
        dupnames1 = {
            item
            for item, count in collections.Counter(names1).items()
            if count > 1
        }
        dupnames2 = {
            item
            for item, count in collections.Counter(names2).items()
            if count > 1
        }
        dupnames = dupnames1.union(dupnames2)
        for i, name1 in enumerate(names1):
            if i in index1_matches:
                continue
            for j, name2 in enumerate(names2):
                if j in index2_matches:
                    continue
                if name1 != name2:
                    continue
                if name1 in dupnames:
                    if formats1[i] != formats2[j]:
                        continue
                if groups1[i] == groups2[j]:
                    index1_matches.append(i)
                    index2_matches.append(j)
        index1_nomatches = [
            i for i, _ in enumerate(ids1) if i not in index1_matches
        ]
        index2_nomatches = [
            i for i, _ in enumerate(ids2) if i not in index2_matches
        ]
        return (
            index1_matches,
            index2_matches,
            index1_nomatches,
            index2_nomatches,
        )
